@@ -349,11 +349,17 @@ class CollectionRequirement:
                     # StrictVersion (x.y.z) and only support pre-releases if an explicit version was set (done above).
                     versions = [v for v in resp if StrictVersion.version_re.match(v)]
             except GalaxyError as err:
-                if err.http_code == 404:
-                    display.vvv("Collection '%s' is not available from server %s %s"
-                                % (collection, api.name, api.api_server))
-                    continue
-                raise
+                if err.http_code != 404:
+                    raise
+
+                versions = []
+
+            # Automation Hub doesn't return a 404 but an empty version list so we check that to align both AH and
+            # Galaxy when the collection is not available on that server.
+            if not versions:
+                display.vvv("Collection '%s' is not available from server %s %s" % (collection, api.name,
+                                                                                    api.api_server))
+                continue
 
             display.vvv("Collection '%s' obtained from server %s %s" % (collection, api.name, api.api_server))
             break
